// netDevice.cpp : R\[ AvP[Vp̃Gg |Cg̒`
//

#include "stdafx.h"
#include <stdio.h>      /* for printf(), fprintf() */
#include <winsock.h>    /* for socket(),... */
#include <stdlib.h>     /* for exit() */

#define MAXPENDING 5    /* Maximum outstanding connection requests */
#define RCVBUFSIZE 32   /* Size of receive buffer */

/*
 *  ubN̒`
 */
typedef struct netblock {
    unsigned char  command;
    unsigned char  size;
    unsigned char  sub;
    unsigned char  dmy;
    unsigned long  cnt;
    unsigned long  arg1;
    union{
       unsigned long  w;
       unsigned short h[2];
       unsigned char  b[4];
    }arg2;
} NETBLOCK;

#define	NETDEVICE_READ	1
#define	NETDEVICE_WRITE	2
#define	NETDEVICE_INT   3
#define	NETDEVICE_COMMAND_MASK	0x1f;

extern "C" void InitializeComSupportModuleN(int * clntSock);
extern "C" void FinalizeComSupportModuleN(void);
extern "C" int DeviceRead(unsigned long address, unsigned long size, void * storage);
extern "C" int DeviceWrite(unsigned long address, unsigned long size, void * storage);
static bool HandlenetDeviceClient(int clntSocket);   /* netDevice client handling function */
static void EnterSendLock(bool * lock);	/* Send NetBlock Lock function */
static void LeaveSendLock(bool * lock);	/* Send NetBlock Unlock funtion */
static void netDeviceError(char *errorMessage);	/* handling error */
void DieWithError(char *errorMessage);  /* Error handling function */

static HANDLE sendMutex = NULL;		    /* Send Mutex */
static DWORD  sendMutexOwner = 0;		/* Send Mutex Owner */
static unsigned long net_count = 0;		/* network counter */
static int clntSock = -1;               /* Socket descriptor for client */

void main(int argc, char *argv[])
{
    int servSock;                    /* Socket descriptor for server */
    struct sockaddr_in nsevServAddr; /* Local address */
    struct sockaddr_in nsevClntAddr; /* Client address */
    unsigned short nsevServPort;     /* Server port */
    int clntLen;                     /* Length of client address data structure */
    WSADATA wsaData;                 /* Structure for WinSock setup communication */

    nsevServPort = 49150;            /* Default Port # */

    if(argc == 2){
        nsevServPort = atoi(argv[1]);  /* first arg:  Local port */
    }
    if (argc != 1)     /* Test for correct number of arguments */
    {
        fprintf(stderr, "Usage:  %s <netDevice Port>\n", argv[0]);
        exit(1);
    }

    if (WSAStartup(MAKEWORD(2, 0), &wsaData) != 0) /* Load Winsock 2.0 DLL */
    {
        fprintf(stderr, "WSAStartup() failed");
        exit(1);
    }

	/* Create Send Mutex */
	sendMutex = CreateMutex(NULL, TRUE, NULL);
	ReleaseMutex(sendMutex);

    /* Create socket for incoming connections */
    if ((servSock = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0)
        netDeviceError("socket() failed");

    /* Construct local address structure */
    memset(&nsevServAddr, 0, sizeof(nsevServAddr));   /* Zero out structure */
    nsevServAddr.sin_family = AF_INET;                /* Internet address family */
    nsevServAddr.sin_addr.s_addr = htonl(INADDR_ANY); /* Any incoming interface */
    nsevServAddr.sin_port = htons(nsevServPort);      /* Local port */

    /* Bind to the local address */
    if (bind(servSock, (struct sockaddr *) &nsevServAddr, sizeof(nsevServAddr)) < 0)
        netDeviceError("bind() failed");

    /* Mark the socket so it will listen for incoming connections */
    if (listen(servSock, MAXPENDING) < 0)
        netDeviceError("listen() failed");

    /* Create separate memory for client argument */
    clntSock = -1;
	printf("NetDeviceServer Ready !!\n");

    for (;;) /* Run forever */
    {
        /* Set the size of the in-out parameter */
        clntLen = sizeof(nsevClntAddr);

        /* Wait for a client to connect */
        if ((clntSock = accept(servSock, (struct sockaddr *) &nsevClntAddr, &clntLen)) < 0)
            netDeviceError("accept() failed");

        /* clntSock is connected to a client! */

        printf("Connect netDevice client %s !\n", inet_ntoa(nsevClntAddr.sin_addr));
		InitializeComSupportModuleN(&clntSock);
        HandlenetDeviceClient(clntSock);
		FinalizeComSupportModuleN();
		printf("Disconnect netDevice client %s !\n", inet_ntoa(nsevClntAddr.sin_addr));
	    closesocket(clntSock);		/* Close client socket */
        clntSock = -1;
    }
    /* NOT REACHED */
}

void SendInterruptMsg(int inhno)
{
	NETBLOCK ibuf;
	int sendMsgSize;
	bool iLock;

	printf("%d", inhno);
	if(clntSock == -1)
		return;
	EnterSendLock(&iLock);
    ibuf.command = NETDEVICE_INT;
    ibuf.size    = sizeof(NETBLOCK);
    ibuf.sub     = 0;
	ibuf.cnt     = htonl(++net_count);
	ibuf.arg1    = htonl(inhno);
	sendMsgSize  = send(clntSock, (char *)&ibuf, sizeof(NETBLOCK), 0);
	LeaveSendLock(&iLock);

}

static bool HandlenetDeviceClient(int clntSocket)
{
	NETBLOCK rbuf;
    char echoBuffer[RCVBUFSIZE];        /* Buffer for echo string */
    int recvMsgSize;                    /* Size of received message */
	int sendMsgSize;
	unsigned long address;
	int i, roff, cmd;
	bool pLock;
	char *b;

    /* Receive message from client */
	roff = 0;
    if ((recvMsgSize = recv(clntSocket, echoBuffer, RCVBUFSIZE, 0)) < 0){
	    fprintf(stderr,"recv(): %d\n", WSAGetLastError());
		return FALSE;
	}

    /* Send received string and receive again until end of transmission */
    while (recvMsgSize > 0)      /* zero indicates end of transmission */
    {
		if((net_count % 10) == 0)
			printf(".");
		for(i = 0 ; recvMsgSize > 0 ; recvMsgSize--){
			b = (char *)&rbuf + roff;
			*b = echoBuffer[i++];
			roff++;
			if(roff >= sizeof(NETBLOCK)){
				address = ntohl(rbuf.arg1);
				cmd = rbuf.command & NETDEVICE_COMMAND_MASK;
				if(cmd == NETDEVICE_READ){
					DeviceRead(address, rbuf.sub, (void *)&rbuf.arg2);
					if(rbuf.sub == 2)
						rbuf.arg2.h[0] = htons(rbuf.arg2.h[0]);
					else if(rbuf.sub == 4)
						rbuf.arg2.w = htonl(rbuf.arg2.w);
				}
				else if(cmd == NETDEVICE_WRITE){
					if(rbuf.sub == 2)
						rbuf.arg2.h[0] = htons(rbuf.arg2.h[0]);
					else if(rbuf.sub == 4)
						rbuf.arg2.w = htonl(rbuf.arg2.w);
					DeviceWrite(address, rbuf.sub, (void *)&rbuf.arg2);
				}
				/* command to client */	
				EnterSendLock(&pLock);
				rbuf.command = (rbuf.command & 0x1f) | 0x20;
				rbuf.cnt = htonl(++net_count);
				sendMsgSize = send(clntSocket, (char *)&rbuf, sizeof(NETBLOCK), 0);
				LeaveSendLock(&pLock);
				if(sendMsgSize != sizeof(NETBLOCK)){
				    fprintf(stderr,"send(): %d\n", WSAGetLastError());
					return FALSE;
				}
				roff = 0;
			}
		}
        /* See if there is more data to receive */
        if ((recvMsgSize = recv(clntSocket, echoBuffer, RCVBUFSIZE,0)) < 0){
		    fprintf(stderr,"recv(): %d\n", WSAGetLastError());
			return FALSE;
		}
	}
	return TRUE;
}

static void EnterSendLock(bool * lock)
{
	if(sendMutex != NULL && sendMutexOwner != GetCurrentThreadId()){
		WaitForSingleObject(sendMutex, INFINITE);
		*lock = TRUE;
		sendMutexOwner = GetCurrentThreadId();
	}
	else{
		*lock = FALSE;
	}
}

static void LeaveSendLock(bool *lock)
{
	if(*lock == TRUE){
		sendMutexOwner = 0;
		if(sendMutex != NULL)
			ReleaseMutex(sendMutex);
	}
}

static void netDeviceError(char *errorMessage)
{
    fprintf(stderr,"%s: %d\n", errorMessage, WSAGetLastError());
	CloseHandle(sendMutex);
    exit(1);
}
