/*
 *  TOPPERS/JSP Educative Program
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Just Standard Profile Kernel
 * 
 *  Copyright (C) 2000-2003 by Embedded and Real-Time Systems Laboratory
 *                              Toyohashi Univ. of Technology, JAPAN
 *  Copyright (C) 2004 by Platform Development Center
 *                              RICOH COMPANY,LTD. JAPAN
 *
 *  L쌠҂́CFree Software Foundation ɂČ\Ă 
 *  GNU General Public License  Version 2 ɋLqĂC
 *  (1)`(4)̏𖞂ꍇɌC{\tgEFAi{\tgEF
 *  Aς̂܂ށDȉjgpEEρEĔzziȉC
 *  pƌĂԁj邱Ƃ𖳏ŋD
 *  (1) {\tgEFA\[XR[ȟ`ŗpꍇɂ́CL̒
 *      \C̗pщL̖ۏ؋K肪Ĉ܂܂̌`Ń\[
 *      XR[hɊ܂܂Ă邱ƁD
 *  (2) {\tgEFAėp\ȃoCiR[hiP[^uIu
 *      WFNgt@C⃉CuȂǁǰ`ŗpꍇɂ́Cp
 *      ɔhLgip҃}jAȂǁjɁCL̒쌠\C
 *      ̗pщL̖ۏ؋Kfڂ邱ƁD
 *  (3) {\tgEFAėps\ȃoCiR[ȟ`܂͋@ɑg
 *      ݍ񂾌`ŗpꍇɂ́Ĉꂩ̏𖞂ƁD
 *    (a) pɔhLgip҃}jAȂǁjɁCL̒
 *        \C̗pщL̖ۏ؋Kfڂ邱ƁD
 *    (b) p̌`ԂCʂɒ߂@ɂāCL쌠҂ɕ񍐂
 *        ƁD
 *  (4) {\tgEFA̗pɂ蒼ړI܂͊ԐړIɐ邢Ȃ鑹
 *      QCL쌠҂Ɛӂ邱ƁD
 * 
 *  {\tgEFÁCۏ؂Œ񋟂Ă̂łDL쌠҂́C
 *  {\tgEFAɊւāC̓Kp\܂߂āCȂۏ؂s
 *  ȂD܂C{\tgEFA̗pɂ蒼ړI܂͊ԐړIɐ
 *  Ȃ鑹QɊւĂC̐ӔC𕉂ȂD
 * 
 *  @(#) $Id: heater.c,v 1.1 2004/10/23 19:22:21 roi Exp $
 */

#include <t_services.h>
#include "kernel_id.h"
#include "akih8pot_device.h"
#include "potti.h"

/*
 *  q[^^XN̏
 */
typedef enum heat_state {
	STATE_STOP_HEATER,			/* q[^~ */
	STATE_BOIL_HEATER,			/*  */
	STATE_KEEP_HEATER,			/* ۉ */
	STATE_ERROR_HEATER			/* G[ */
} HEAT_STATE;

/*
 *  ڕWxe[u
 */
static H temp_table[3][2] = {
	100*8-2, 98*8,				/* ۉ[hF */
	100*8-2, 90*8,				/* ۉ[hFߖ */
	100*8-2, 60*8				/* ۉ[hF~N */
};

static UH pottemp;
static H  targettemp = 0;

/*
 *  q[^^XN
 *  q[^̊Ǘsp^XN
 */
void
heater_task(VP_INT exinf)
{
	SYSTIM     boil_time;
	ER         ercd;
	FLGPTN     flgptn;
	HEAT_STATE state = STATE_STOP_HEATER;
	H          oldtemp;
	UB         potlevel, oldlevel;
	int        count;

	syslog(LOG_INFO, "Pot heater task starts (exinf = %d).", exinf);
	pottemp  = get_temp();			/* Ԃ̉xݒ */
	oldtemp  = pottemp - 1;
	potlevel = get_level();			/* Ԃ̐ʂ̐ݒ */
	oldlevel = potlevel - 1;
	count  = 0;						/* JE^̏ */
	set_control(CTL_HEATPOWER, ON);	/* q[^d */
	clr_flg(HEAT_FLGID, 0);
	for(;;){
		ercd = twai_flg(HEAT_FLGID, EVT_HEAT, TWF_ORW, &flgptn, HEATER_CYCLE);
		if(ercd == E_OK && state != STATE_ERROR_HEATER){
			clr_flg(HEAT_FLGID, ~flgptn);
			syslog(LOG_INFO, "get HEAT_FLGID=0x%x !\n", flgptn);
			if(flgptn & HEAT_ERROR){
				state = STATE_ERROR_HEATER;
				targettemp = 0;
				set_flg(TIM_FLGID, TIM_30SEC_BUZZER);
			}
			else if(potlevel > 15 || potlevel == 0 || (potstate & COVER_OPEN)){
				state = STATE_STOP_HEATER;
				targettemp = 0;
			}
			else{	/* Aۉ̕ύX */
				unsigned int mode;

				mode = (potstate & MODE_MASK) - 1;
				if(state == STATE_BOIL_HEATER){
					if(flgptn & HEAT_BOILCHG){
						targettemp = temp_table[mode][1];
						state = STATE_KEEP_HEATER;
					}
				}
				else{
					if(flgptn & (HEAT_BOIL|HEAT_BOILCHG)){
						targettemp = temp_table[mode][0];
						state = STATE_BOIL_HEATER;
						boil_time = 0;
					}
					else{
						targettemp = temp_table[mode][1];
						state = STATE_KEEP_HEATER;
					}
				}
			}
		}
		switch(state){
		case STATE_STOP_HEATER:		/* q[^~ */
			syslog(LOG_INFO, "change STATE_STOP_HEATER");
			set_control(CTL_HEATER, OFF);
			set_led(LED_BOIL, OFF);
			set_led(LED_KEEPWARM, OFF);
			break;
		case STATE_BOIL_HEATER:		/*  */
			syslog(LOG_INFO, "change STATE_BOIL_HEATER");
			set_led(LED_BOIL, ON);
			set_led(LED_KEEPWARM, OFF);
			set_control(CTL_HEATER, ON);
			if(boil_time){
				if((get_simtime() - boil_time) >= 30*60){
					state = STATE_KEEP_HEATER;
					set_flg(HEAT_FLGID, HEAT_CHECK);
				}
			}
			else if(pottemp >= targettemp)
				boil_time = get_simtime();
			break;
		case STATE_KEEP_HEATER:		/* ۉ */
			syslog(LOG_INFO, "change STATE_KEEP_HEATER");
			set_led(LED_BOIL, OFF);
			set_led(LED_KEEPWARM, ON);
			if(pottemp > targettemp)
				set_control(CTL_HEATER, OFF);
			else if(pottemp < targettemp)
				set_control(CTL_HEATER, ON);
			break;
		default:					/* G[ */
			syslog(LOG_INFO, "change STATE_ERROR_HEATER");
			set_control(CTL_HEATER, OFF);
			set_control(CTL_HEATPOWER, OFF);
			set_led(LED_BOIL, OFF);
			set_led(LED_KEEPWARM, OFF);
			break;
		}

		pottemp = get_temp();		/* x̎o */
		count++;					/* JEgAbv */
		if(count & 1){				/* 200ms̏ */
			potlevel = get_level();
			set_led(LED_LEVEL, potlevel);
		}
		if(pottemp != oldtemp || potlevel != oldlevel){
			set_flg(HEAT_FLGID, HEAT_CHECK);
			display_temp((pottemp+5)>>3, potlevel);
			oldtemp  = pottemp;
			oldlevel = potlevel;
		}
	}
}

/*
 *  G[^XN
 */
void
error_task(VP_INT exinf)
{
	SYSTIM check_time;
	H      check_temp;

	for(;;){
		tslp_tsk(TEST_CYCLE);
		if(pottemp >= 110*8)		/* G[ */
			set_flg(HEAT_FLGID, HEAT_ERROR);
		if(targettemp && pottemp < (targettemp - 5*8)){
			if(check_time != 0){	/* xオ炸G[ */
				if((get_simtime() - check_time) > 60*10)
					set_flg(HEAT_FLGID, HEAT_ERROR);
				else if(pottemp > check_temp)
					check_time = 0;
			}
			else{
				check_time = get_simtime();
				check_temp = pottemp;
			}
		}
		else
			check_time = 0;
	}
}

